--the script checks the database format
--and updates it to the latest version if necessary
--run it against the Helpdesk's database to upgrade it!
--PS. most recent changes are at the bottom... top-to-bottom order.


--###########################################################################

--fileAttachments table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdFileAttachments]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdFileAttachments](
		[FileID] [int] IDENTITY(1,1) PRIMARY KEY NOT NULL,
		[IssueID] [int] NOT NULL,
		[UserID] [int] NOT NULL,
		[FileName] [varchar](255) NOT NULL,
		[FileData] [image] NULL
	)
END
GO

IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdFileAttachments_hdUsers]'))
ALTER TABLE hdFileAttachments WITH CHECK ADD CONSTRAINT [FK_hdFileAttachments_hdUsers] FOREIGN KEY([UserID])
REFERENCES hdUsers (UserID)
GO
ALTER TABLE hdFileAttachments CHECK CONSTRAINT [FK_hdFileAttachments_hdUsers]
GO
--fileattachmetns table finished

--allowUserRegistration column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowUserRegistration')
begin
	  ALTER TABLE hdSettings ADD AllowUserRegistration bit NOT NULL DEFAULT 1
end

--###########add MailChecker columns
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerEnabled')
begin
	  ALTER TABLE hdSettings ADD MailCheckerEnabled bit NOT NULL DEFAULT 0
end
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerInterval')
begin
	  ALTER TABLE hdSettings ADD MailCheckerInterval int NOT NULL DEFAULT 600
end
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerNewIssuesCategoryID')
begin
	  ALTER TABLE hdSettings ADD MailCheckerNewIssuesCategoryID int NOT NULL DEFAULT (1)
end
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerAllowUnregisteredUsers')
begin
	  ALTER TABLE hdSettings ADD MailCheckerAllowUnregisteredUsers bit NOT NULL DEFAULT (0)
end
--#############

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='Priority')
begin
	  ALTER TABLE hdIssues ADD Priority smallint NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPPort')
begin
	  ALTER TABLE hdSettings ADD SMTPPort int NOT NULL DEFAULT 25
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SMTPUseSSL')
begin
	  ALTER TABLE hdSettings ADD SMTPUseSSL bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictIssueDeletion')
begin
	  ALTER TABLE hdSettings ADD RestrictIssueDeletion bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MaxAttachSize')
begin
	  ALTER TABLE hdSettings ADD MaxAttachSize int NOT NULL DEFAULT (1000)
end


if not exists (select * from syscolumns where id=object_id('hdSettings') and name='Lang')
begin
	  ALTER TABLE hdSettings ADD Lang [varchar] (10) NOT NULL DEFAULT ('en-US')
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='FirstName')
begin
	  ALTER TABLE hdUsers ADD FirstName varchar(255) NOT NULL DEFAULT ('')
end
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='LastName')
begin
	  ALTER TABLE hdUsers ADD LastName varchar(255) NOT NULL DEFAULT ('')
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='RestrictIssueClosing')
begin
	  ALTER TABLE hdSettings ADD RestrictIssueClosing bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='EmailTemplate')
begin
	  ALTER TABLE hdSettings ADD EmailTemplate varchar(2000) NOT NULL DEFAULT ('')
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Notes')
begin
	  ALTER TABLE hdUsers ADD Notes varchar(4000) NOT NULL DEFAULT ('')
end

--Sections table		  
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdSections]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdSections] (
	[SectionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
	[Name] [varchar] (50) NOT NULL
	)
END

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='SectionID')
begin
	  ALTER TABLE hdCategories ADD SectionID int NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NewIssuesDefaultCategoryID')
begin
	  ALTER TABLE hdSettings ADD NewIssuesDefaultCategoryID int NOT NULL DEFAULT 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Location')
begin
	  ALTER TABLE hdUsers ADD Location varchar(1000) NULL
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Phone')
begin
	  ALTER TABLE hdUsers ADD Phone varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='KBForTechsOnly')
begin
	  ALTER TABLE hdIssues ADD KBForTechsOnly bit NOT NULL DEFAULT 0
end

if not exists (select * from syscolumns where id=object_id('hdComments') and name='ForTechsOnly')
begin
	  ALTER TABLE hdComments ADD ForTechsOnly bit NOT NULL DEFAULT 0
end


/*#####!!!multiinstance changes start #################*/

--if hdInstances - rename
IF EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdInstances') AND type in (N'U'))
begin
	exec sp_rename 'hdInstances', 'Instances'
end

--InstanceID column
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='InstanceID')
begin
	  ALTER TABLE hdSettings ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdCategories] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdSections] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdIssues] ADD InstanceID int NOT NULL DEFAULT 0
	  ALTER TABLE [hdUsers] ADD InstanceID int NOT NULL DEFAULT 0
end
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[Instances]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[Instances] (
		[InstanceID] int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[Name] [varchar] (100) NOT NULL,
		[CustomerID] int NOT NULL,
		ValidTill datetime NOT NULL
	)
END
GO

--drop unique user name constraint
if exists (select * from sysobjects where name = 'IX_hdUsers_UniqueName')
begin
	alter table hdUsers drop constraint IX_hdUsers_UniqueName
	
	--add username/instanceid uniqeness
	if not exists (select * from sysobjects where name = 'IX_hdUsers_UniqueNameInstance')
	begin
		ALTER TABLE [dbo].[hdUsers] WITH NOCHECK ADD 
		CONSTRAINT [IX_hdUsers_UniqueNameInstance] UNIQUE  NONCLUSTERED 
		(
			[username],
			InstanceID
		)
	end
end
/*##################!!!!!!!!!!end of multiinstance code*/
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='UseDefaultSmtp')
begin
	  ALTER TABLE hdSettings ADD UseDefaultSmtp bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowAnonymousKBAccess')
begin
	  ALTER TABLE hdSettings ADD AllowAnonymousKBAccess bit not null default 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AllowGuestSubmission')
begin
	  ALTER TABLE hdSettings ADD AllowGuestSubmission bit not null default 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='SendEmail')
begin
	  ALTER TABLE hdUsers ADD SendEmail bit not null default 1
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HeaderBGColor')
begin
	  ALTER TABLE hdSettings ADD HeaderBGColor char(6) NOT NULL DEFAULT('000000')
	  ALTER TABLE hdSettings ADD MenuBarBGColor char(6) NOT NULL DEFAULT('FF7700')
	  ALTER TABLE hdSettings ADD LogoImage [image] NULL
end
GO
	
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SeeEveryonesIssues')
begin
	  ALTER TABLE hdSettings ADD SeeEveryonesIssues bit NOT NULL DEFAULT 0
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='EmailSubjectTemplate')
begin
	  ALTER TABLE hdSettings ADD EmailSubjectTemplate varchar (1000) NOT NULL DEFAULT ''
end
GO

--enlarge the password field so MD5-string fits there
if exists(select name from syscolumns where id=object_id('hdUsers') and name='Password' and length<50)
begin
	alter table hdUsers alter column Password [varchar](50) NOT NULL
end

--dropping IX_hdUsers_UniqueEmailInstance constraint - not needed.
if exists (select * from sysobjects where name = 'IX_hdUsers_UniqueEmailInstance')
begin
	alter table hdUsers drop constraint IX_hdUsers_UniqueEmailInstance
end
GO

--converting Body and Subject and Status from varchar to Nvarchar
if exists (select id from syscolumns where id=object_id('hdIssues') and name='Subject' and type_name(xtype)<>'nvarchar')
	alter table hdIssues alter column Subject nvarchar(1000) NOT NULL
if exists (select id from syscolumns where id=object_id('hdStatus') and name='Name' and type_name(xtype)<>'nvarchar')
	alter table hdStatus alter column [Name] nvarchar (50) NOT NULL
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SendTechniciansEmailOnNewIssue')
begin
	ALTER TABLE hdSettings ADD SendTechniciansEmailOnNewIssue bit NOT NULL DEFAULT 1
end

--adding hdNews table
if not exists (select * from INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='hdNews')
begin
	CREATE TABLE [dbo].[hdNews] (
		[NewsID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[NewsDate] [datetime] NOT NULL DEFAULT (getdate()),
		[UserID] [int] NOT NULL ,
		[Subject] [nvarchar] (100) NOT NULL ,
		[Body] [nvarchar] (4000) NOT NULL
	)
end
GO

--adding "Custom Fields" functionality
if not exists (select * from INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='hdCustomFields')
begin
	CREATE TABLE [dbo].[hdCustomFields] (
		[FieldID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] [int] NOT NULL,
		[Type] [int] NOT NULL,
		[FieldName] nvarchar(100) NOT NULL
	)
	CREATE TABLE [dbo].[hdCustomFieldValues] (
		[IssueID] [int] NOT NULL,
		[FieldID] [int] NOT NULL,
		[Value] sql_variant NOT NULL
	)
	ALTER TABLE [dbo].[hdCustomFieldValues] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdCustomFieldValues]PRIMARY KEY CLUSTERED 
		(
			[IssueID],
			[FieldID]
		)
	CREATE TABLE [dbo].[hdCustomFieldOptions] (
		[OptionID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[FieldID] [int] NOT NULL,
		[OptionValue] nvarchar(500) NOT NULL
	)
end
GO


if not exists (select * from syscolumns where id=object_id('hdNews') and name='InstanceID')
begin
	ALTER TABLE hdNews ADD InstanceID int NOT NULL DEFAULT(0)
end

--enlarging username and email columns
if exists(select name from syscolumns where id=object_id('hdUsers') and name='UserName' and length<100)
	alter table hdUsers alter column UserName varchar(100) NOT NULL
if exists(select name from syscolumns where id=object_id('hdUsers') and name='Email' and length<100)
	alter table hdUsers alter column Email varchar(100) NOT NULL

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ServerTimeOffset')
begin
	ALTER TABLE hdSettings ADD ServerTimeOffset int NOT NULL DEFAULT 0
end

--adding index to hdComments (IssueID
if not exists (select name FROM sysindexes WHERE name = 'IX_hdComments_IssueID') begin
	CREATE INDEX IX_hdComments_IssueID ON hdComments (IssueID)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='ReplyToAddress')
begin
	ALTER TABLE hdSettings ADD ReplyToAddress varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='StartDate')
begin
	ALTER TABLE hdIssues ADD StartDate datetime NULL
end

--hdIssueSubscribers table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'[dbo].[hdIssueSubscribers]') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdIssueSubscribers] (
		[IssueID] [int] NOT NULL,
		[UserID] [int] NOT NULL
	)
END
GO
IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdIssueSubscribers_hdIssues]'))
ALTER TABLE hdIssueSubscribers WITH CHECK ADD CONSTRAINT [FK_hdIssueSubscribers_hdIssues] FOREIGN KEY([IssueID])
REFERENCES hdIssues (IssueID)
GO
ALTER TABLE hdIssueSubscribers CHECK CONSTRAINT [FK_hdIssueSubscribers_hdIssues]
GO
IF NOT EXISTS (SELECT * FROM sysforeignkeys WHERE constid = OBJECT_ID(N'[dbo].[FK_hdIssueSubscribers_hdUsers]'))
ALTER TABLE hdIssueSubscribers WITH CHECK ADD CONSTRAINT [FK_hdIssueSubscribers_hdUsers] FOREIGN KEY([UserID])
REFERENCES hdUsers (UserID)
GO
ALTER TABLE hdIssueSubscribers CHECK CONSTRAINT [FK_hdIssueSubscribers_hdUsers]
GO
--hdIssueSubscribers table finished

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='LastUpdated')
begin
	  ALTER TABLE hdIssues ADD LastUpdated datetime NOT NULL DEFAULT(getdate())
end

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='ButtonCaption')
begin
	  ALTER TABLE hdStatus ADD ButtonCaption nvarchar(50) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdStatus') and name='InstanceID')
begin
	  ALTER TABLE hdStatus ADD InstanceID int NULL
end

--converting from nvarchar(4000) to Nvarchar(MAX)
if exists(select name from syscolumns where id=object_id('hdNews') and name='Body' and length=4000)
	alter table hdNews alter column Body nvarchar(MAX) NOT NULL
if exists(select name from syscolumns where id=object_id('hdIssues') and name='Body' and length=4000)
	alter table hdIssues alter column Body nvarchar(MAX) NOT NULL
if exists(select name from syscolumns where id=object_id('hdComments') and name='Body' and length=4000)
	alter table hdComments alter column Body nvarchar(MAX) NOT NULL
GO

if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='ForTechsOnly')
begin
	  ALTER TABLE hdCustomFields ADD ForTechsOnly bit NOT NULL DEFAULT (0)
end

--asset management tables
if not exists (select * from INFORMATION_SCHEMA.TABLES WHERE TABLE_NAME='hdAssets')
begin
	CREATE TABLE [dbo].[hdAssets] (
		ItemID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		ModelName nvarchar(1000) NOT NULL,
		TypeID int NOT NULL,
		ManufacturerID int NOT NULL,
		Quantity int NOT NULL DEFAULT(0),
		SupplierID int NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetManufacturers] (
		ManufacturerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetSuppliers] (
		SupplierID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetTypes] (
		TypeID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		[Name] nvarchar(1000) NOT NULL
	)
	CREATE TABLE [dbo].[hdAssetUsers] (
		ItemID int NOT NULL,
		UserID int NOT NULL
	)
	ALTER TABLE [dbo].[hdAssetUsers] WITH NOCHECK ADD 
		CONSTRAINT [PK_hdAssetUsers] PRIMARY KEY CLUSTERED ([ItemID],[UserID])
end
GO

--adding [Mandatory] and [OrderByNumber] to hdCustomFields
if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='Mandatory')
begin
	ALTER TABLE hdCustomFields ADD [Mandatory] bit NOT NULL DEFAULT (0)
	ALTER TABLE hdCustomFields ADD [OrderByNumber] int NOT NULL DEFAULT (100)
end
GO

if exists (select * from syscolumns where id=object_id('hdSettings') and name='SendHtmlEmails')
begin
	--dropping default constraints and then columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'SendHtmlEmails')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	ALTER TABLE hdSettings DROP COLUMN SendHtmlEmails
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='FromName')
begin
	  ALTER TABLE hdSettings ADD FromName varchar(50) NULL
end

--more assets fields
if not exists (select * from syscolumns where id=object_id('hdAssets') and name='SerialNumber')
begin
	ALTER TABLE hdAssets ADD
		SerialNumber nvarchar(200) NULL,
		Location nvarchar(500) NULL,
		Comments nvarchar(1000) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableKB')
begin
	  ALTER TABLE hdSettings ADD DisableKB bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AutoLoginSharedSecret')
begin
	  ALTER TABLE hdSettings ADD AutoLoginSharedSecret varchar(20) NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='MenuBarTextColor')
begin
	  ALTER TABLE hdSettings ADD MenuBarTextColor char(6) NOT NULL DEFAULT('C7D4E2')
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Disabled')
begin
	  ALTER TABLE hdUsers ADD Disabled bit not null default 0
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableCloseNotification')
begin
	  ALTER TABLE hdSettings ADD DisableCloseNotification bit NOT NULL DEFAULT 1
end

if not exists (select * from syscolumns where id=object_id('hdCategories') and name='ForTechsOnly')
begin
	ALTER TABLE hdCategories ADD  ForTechsOnly bit NOT NULL DEFAULT (0)
end

--adding commant teamplets table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCommentTemplates') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdCommentTemplates] (
		[TemplateID] [int] IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[Body] nvarchar(max) NOT NULL ,
	)
END
GO

--converting filenames to nvarchar from varchar
if exists(select name from syscolumns where id=object_id('hdFileAttachments') and name='FileName' and length<255)
	alter table hdFileAttachments alter column FileName nvarchar(255) NOT NULL

--adding "Companies" table and moving data to it
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdCompanies') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdCompanies] (
		CompanyID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[Name] nvarchar(255) NOT NULL
	)
	
	ALTER TABLE hdUsers ADD CompanyID int NULL
	
	if exists (select * from syscolumns where id=object_id('hdUsers') and name='Company')
	begin
		declare @sql nvarchar(max)
		set @sql = N'INSERT INTO hdCompanies (InstanceID, [Name])
		SELECT DISTINCT InstanceID, Company FROM hdUsers WHERE Company IS NOT NULL AND RTRIM(Company)<>'''''
		exec sp_executesql @sql
	
		set @sql = N'UPDATE hdUsers SET CompanyID = hdCompanies.CompanyID
		from hdUsers
		inner join hdCompanies ON hdUsers.Company=hdCompanies.Name
		where hdUsers.Company IS NOT NULL'
		exec sp_executesql @sql
	end
END
GO

if exists (select * from syscolumns where id=object_id('hdUsers') and name='Company')
begin
	ALTER TABLE hdUsers DROP COLUMN Company
end

--adding "POPServers" table and moving data to it
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdPOPServers') AND type in (N'U'))
BEGIN
	CREATE TABLE hdPOPServers (
		ServerID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		HostName varchar (50) NULL,
		Port int NOT NULL DEFAULT (110),
		POPLogin varchar (50) NULL,
		POPPassword varchar (50) NULL,
		UseSSL bit NOT NULL DEFAULT (0)
	)
	declare @sql nvarchar(max)
	set @sql = N'INSERT INTO hdPOPServers (InstanceID, HostName, Port, POPLogin, POPPassword, UseSSL)
	SELECT InstanceID, MailCheckerPOPServer, MailCheckerPOPPort, MailCheckerPOPLogin, MailCheckerPOPPassword, MailCheckerUseSSL
	FROM hdSettings
	WHERE MailCheckerPOPServer IS NOT NULL AND MailCheckerPOPServer<>'''''
	exec sp_executesql @sql
END
GO
--removing unneeded POP cols from hdsettings
if exists (select * from syscolumns where id=object_id('hdSettings') and name='MailCheckerPOPServer') begin
	--dropping default constraints and then columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'MailCheckerPOPPort')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'MailCheckerUseSSL')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	ALTER TABLE hdSettings DROP COLUMN MailCheckerPOPServer, MailCheckerPOPPort, MailCheckerPOPLogin, MailCheckerPOPPassword, MailCheckerUseSSL
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='NewTicketsCategoryID')
begin
	ALTER TABLE hdPOPServers ADD NewTicketsCategoryID int NULL
end

if not exists (select * from syscolumns where id=object_id('hdComments') and name='IsSystem')
begin
	ALTER TABLE hdComments ADD IsSystem bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('Instances') and name='CustomDomain')
begin
	ALTER TABLE Instances ADD CustomDomain varchar(100) NULL
end
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='Title')
begin
	ALTER TABLE hdSettings ADD Title varchar(500) NOT NULL DEFAULT ''
end
GO
--moving from instances if any
if exists (select * from syscolumns where id=object_id('Instances') and name='Description')
begin
	declare @sql nvarchar(max)
	set @sql = N'update hdSettings SET hdSettings.Title = Instances.Description
	from Instances inner join hdSettings ON Instances.InstanceID=hdSettings.InstanceID'
	exec sp_executesql @sql

	ALTER TABLE Instances DROP COLUMN Description
end

--adding hdLinkedIssues table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdLinkedIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE hdLinkedIssues (
		Issue1ID int NOT NULL,
		Issue2ID int NOT NULL
	)
	ALTER TABLE hdLinkedIssues WITH NOCHECK ADD 
		CONSTRAINT [PK_hdLinkedIssues] PRIMARY KEY CLUSTERED 
		([Issue1ID],[Issue2ID])
END
GO

--adding hdIssueSchedule table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdIssueSchedule') AND type in (N'U'))
BEGIN
	CREATE TABLE hdIssueSchedule (
		[IssueID] int NOT NULL PRIMARY KEY,
		FreqType int NOT NULL,
		FirstRecurrenceDate datetime NOT NULL,
		PrevRepeatsCount int NOT NULL DEFAULT(0)
	)
	CREATE TABLE hdScheduledIssues (
		ScheduledIssueID int NOT NULL PRIMARY KEY,
		OriginalIssueID int NOT NULL
	)
END
GO

--adding "OrderByNumber" to hdCategories
if not exists (select * from syscolumns where id=object_id('hdCategories') and name='OrderByNumber')
begin
	ALTER TABLE hdCategories ADD OrderByNumber int NOT NULL DEFAULT 1
	ALTER TABLE hdSections ADD OrderByNumber int NOT NULL DEFAULT 1
end
GO

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='LastSeen')
begin
	ALTER TABLE hdUsers ADD LastSeen datetime null
end

if not exists (select * from syscolumns where id=object_id('hdComments') and name='HiddenFromKB')
begin
	ALTER TABLE hdComments ADD HiddenFromKB bit NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableAssets')
begin
	ALTER TABLE hdSettings ADD DisableAssets bit not null default 0
end

--adding hdMergedIssues table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdMergedIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE hdMergedIssues (
		[IssueID] int NOT NULL,
		[IssueMergedID] int NOT NULL
	)
	ALTER TABLE hdMergedIssues WITH NOCHECK ADD CONSTRAINT [PK_hdMergedIssues] PRIMARY KEY CLUSTERED (IssueID, IssueMergedID)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DaysToCloseAfter')
begin
	ALTER TABLE hdSettings ADD DaysToCloseAfter int NOT NULL DEFAULT(0)
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='IPAddress')
begin
	ALTER TABLE hdUsers ADD IPAddress varchar(50) null
	ALTER TABLE hdUsers ADD HostName varchar(255) null
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Lang')
begin
	ALTER TABLE hdUsers ADD Lang varchar(10)  NOT NULL DEFAULT('')
end

if not exists (select * from syscolumns where id=object_id('hdPOPServers') and name='IsImap')
begin
	  ALTER TABLE hdPOPServers ADD IsImap bit NOT NULL DEFAULT 0
end

--adding indexes to hdIssues
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_CategoryID') begin
	CREATE INDEX IX_hdIssues_CategoryID ON hdIssues (CategoryID)
end
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_UserID') begin
	CREATE INDEX IX_hdIssues_UserID ON hdIssues (UserID)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='CRMRootUrl')
begin
	  ALTER TABLE hdSettings ADD CRMRootUrl varchar(1000) not null DEFAULT ''
	  ALTER TABLE hdSettings ADD CRMSharedKey varchar(255) not null DEFAULT ''

	  ALTER TABLE hdCompanies ADD CRMID int NULL
end

if not exists (select * from syscolumns where id=object_id('hdUsers') and name='UserAgent')
begin
	ALTER TABLE hdUsers ADD UserAgent varchar(255) null
end

if not exists (select * from syscolumns where id=object_id('hdIssues') and name='DueDate')
begin
	ALTER TABLE hdIssues ADD DueDate datetime NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='UseFromNameForUserOriginatedMsgs')
begin
	ALTER TABLE hdSettings ADD UseFromNameForUserOriginatedMsgs bit NOT NULL default(0)
end

if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='CommentID')
begin
	ALTER TABLE hdFileAttachments ADD CommentID int NOT NULL DEFAULT (0)
end

--TimeSpentInSeconds col
if not exists (select * from syscolumns where id=object_id('hdIssues') and name='TimeSpentInSeconds')
begin
	ALTER TABLE hdIssues ADD TimeSpentInSeconds int NOT NULL DEFAULT (0)
end
GO
if exists (select * from syscolumns where id=object_id('hdIssues') and name='HoursSpent')
begin
	exec sp_executesql N'UPDATE hdIssues SET TimeSpentInSeconds=CAST(HoursSpent * 3600 as int)'
	
	--dropping default constraints and then columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdIssues') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdIssues') and name = 'HoursSpent')
	if not @default is null
	begin
		set @sql = N'alter table hdIssues drop constraint ' + @default
		exec sp_executesql @sql
	end
	
	ALTER TABLE hdIssues DROP COLUMN HoursSpent
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HeaderTextColor')
begin
	ALTER TABLE hdSettings ADD HeaderTextColor char(6) NOT NULL DEFAULT('FFFFFF')
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SameCompanyUserPermission')
begin
	ALTER TABLE hdSettings ADD SameCompanyUserPermission bit not null default(0)
end

--adding indexes to hdIssues
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_StatusID') begin
	CREATE INDEX IX_hdIssues_StatusID ON hdIssues (StatusID)
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='HidePoweredBy')
begin
	ALTER TABLE hdSettings ADD HidePoweredBy bit not null default(0)
end

--adding hdMergedIssues table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdOnBehalfIssues') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdOnBehalfIssues] (
		[IssueID] [int] NOT NULL PRIMARY KEY,
		[SubmittedByUserID] int NOT NULL
	)
END
GO

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NewIssueEmailSubject')
begin
	ALTER TABLE hdSettings ADD NewIssueEmailSubject varchar(500) NOT NULL DEFAULT ''
	ALTER TABLE hdSettings ADD NewIssueEmailTemplate varchar(2000) NOT NULL DEFAULT ''
end

--TAGS!
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdTags') AND type in (N'U'))
BEGIN
	CREATE TABLE [dbo].[hdTags] (TagID int IDENTITY (1, 1) NOT NULL PRIMARY KEY, [Name] nvarchar(200) NOT NULL, InstanceID int NOT NULL)

	CREATE TABLE [dbo].[hdIssueTags] (TagID int NOT NULL, IssueID int NOT NULL)
	ALTER TABLE hdIssueTags WITH NOCHECK ADD CONSTRAINT [PK_hdIssueTags] PRIMARY KEY CLUSTERED (TagID,IssueID)
END

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableAvatars')
begin
	ALTER TABLE hdSettings ADD DisableAvatars bit not null default(0)
end

--UpdatedForTechView col
if not exists (select * from syscolumns where id=object_id('hdIssues') and name='UpdatedForTechView')
begin
	ALTER TABLE hdIssues ADD UpdatedForTechView bit NOT NULL DEFAULT (0)
end

--not an upgrade but a maintenance procedure: remove deleted tickets from the "hdOnBehalfIssues" tbl
delete from hdOnBehalfIssues where IssueID NOT IN (SELECT IssueID FROM hdIssues)

--Name col in CommentTemplates
if not exists (select * from syscolumns where id=object_id('hdCommentTemplates') and name='Name')
begin
	ALTER TABLE hdCommentTemplates ADD Name varchar(50) NULL
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SendAdminEmailOnNewIssue')
begin
	ALTER TABLE hdSettings ADD SendAdminEmailOnNewIssue bit NOT NULL DEFAULT 1
end

if not exists (select * from syscolumns where id=object_id('hdSettings') and name='SendTechSMSOnNewIssue')
begin
	ALTER TABLE hdSettings ADD SendTechSMSOnNewIssue bit NOT NULL DEFAULT 0
end

--adding index to hdFileAttachments (IssueID)
if not exists (select name FROM sysindexes WHERE name = 'IX_hdFileAttachments_IssueID') begin
	CREATE INDEX IX_hdFileAttachments_IssueID ON hdFileAttachments (IssueID)
end

--deprecated setting
if exists (select * from syscolumns where id=object_id('hdSettings') and name='DisableCaptcha')
begin
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D'
		AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'DisableCaptcha')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end

	ALTER TABLE hdSettings DROP COLUMN DisableCaptcha
end

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'hdAssetsIssues') AND type in (N'U'))
begin
	CREATE TABLE dbo.hdAssetsIssues	(IssueID int NOT NULL, AssetID int NOT NULL)
	ALTER TABLE [dbo].hdAssetsIssues WITH NOCHECK ADD CONSTRAINT [PK_hdAssetsIssues] PRIMARY KEY CLUSTERED ([IssueID], [AssetID]) 
end

--Updating default color from the old ones to the new ones, not changing if user has custom colors
IF EXISTS (SELECT * FROM hdSettings WHERE HeaderBGColor = '000000' AND InstanceID = 0)begin
UPDATE hdSettings SET HeaderBGColor ='34383E' WHERE InstanceID = 0
end

IF EXISTS (SELECT * FROM hdSettings WHERE MenuBarBGColor = 'FF7700' AND InstanceID = 0)begin
UPDATE hdSettings SET MenuBarBGColor ='4C5B6B' WHERE InstanceID = 0
end

IF EXISTS (SELECT * FROM hdSettings WHERE MenuBarTextColor = 'FFFFFF' AND InstanceID = 0)begin
UPDATE hdSettings SET MenuBarTextColor ='C7D4E2' WHERE InstanceID = 0
end

if exists(select name from syscolumns where id=object_id('hdIssues') and name='Subject' and length<1000)
	ALTER TABLE hdIssues ALTER COLUMN Subject nvarchar(1000)

GO

--passwords to md5 (security)
UPDATE hdUsers SET [Password]=LOWER(CONVERT(VARCHAR(32), HashBytes('MD5', [Password]), 2))
WHERE len([Password])<32

if not exists (select * from syscolumns where id=object_id('hdCustomFields') and name='ShowInGrid')
begin
	  ALTER TABLE hdCustomFields ADD ShowInGrid bit NOT NULL DEFAULT (1)
end

--signature field for users
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Signature')
begin
	  ALTER TABLE hdUsers ADD [Signature] nvarchar(500) null
end

--new setting
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='AutoAssignFirstReplier')
begin
	ALTER TABLE hdSettings ADD AutoAssignFirstReplier bit NOT NULL DEFAULT 1
end

--new settings - DontReopenOnReply
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='DontReopenOnReply')
begin
	ALTER TABLE hdSettings ADD DontReopenOnReply bit NOT NULL DEFAULT 0
end

--NotifyAllTechsOnReply
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NotifyAllTechsOnReply')
begin
	ALTER TABLE hdSettings ADD NotifyAllTechsOnReply bit NOT NULL DEFAULT 0
end

--increase comment template name size
alter table hdCommentTemplates alter column [Name] varchar(255) NULL

IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'UserAvatars') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[UserAvatars] ([UserID] [int] NOT NULL PRIMARY KEY, [ImageData] [image] NULL)
end

--Recipients
if not exists (select * from syscolumns where id=object_id('hdComments') and name='Recipients')
begin
	ALTER TABLE hdComments ADD Recipients nvarchar(2000) NOT NULL DEFAULT ''
end

--GoogleDrivePath
if not exists (select * from syscolumns where id=object_id('hdFileAttachments') and name='GoogleDriveUrl')
begin
	ALTER TABLE hdFileAttachments ADD GoogleDriveUrl varchar(255) NULL
end

--make FileData column nullable
ALTER TABLE hdFileAttachments ALTER COLUMN FileData [image] NULL

--NewTicketNote
if not exists (select * from syscolumns where id=object_id('hdSettings') and name='NewTicketNote')
begin
	ALTER TABLE hdSettings ADD NewTicketNote varchar(max) NULL
end

GO

--move email templates to separate table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'EmailTemplates') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[EmailTemplates] (
		[InstanceID] int NOT NULL DEFAULT(0),
		EmailTemplate varchar (3000) NOT NULL DEFAULT '',
		EmailSubjectTemplate varchar (500) NOT NULL DEFAULT '',
		NewIssueEmailSubject varchar(500) NOT NULL DEFAULT '',
		NewIssueEmailTemplate varchar(3000) NOT NULL DEFAULT '',
		WelcomeEmailSubj varchar (500) NOT NULL DEFAULT '',
		WelcomeEmailBody varchar (3000) NOT NULL DEFAULT '',
		NewIssueConfirmationSubj varchar(500) NOT NULL DEFAULT '',
		NewIssueConfirmationBody varchar(3000) NOT NULL DEFAULT ''
	)

	INSERT INTO EmailTemplates (InstanceID, EmailTemplate, EmailSubjectTemplate, NewIssueEmailSubject, NewIssueEmailTemplate, NewIssueConfirmationSubj, NewIssueConfirmationBody)
	SELECT InstanceID, EmailTemplate, EmailSubjectTemplate, NewIssueEmailSubject, NewIssueEmailTemplate, NewIssueEmailSubject, NewIssueEmailTemplate FROM hdSettings

	--droping defaults then dropping columns
	declare @default sysname, @sql nvarchar(max)
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'EmailTemplate')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'EmailSubjectTemplate')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'NewIssueEmailSubject')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end
	select @default = name from sys.default_constraints where parent_object_id = object_id('hdSettings') AND type = 'D' AND parent_column_id = (select column_id from sys.columns where object_id = object_id('hdSettings') and name = 'NewIssueEmailTemplate')
	if not @default is null
	begin
		set @sql = N'alter table hdSettings drop constraint ' + @default
		exec sp_executesql @sql
	end
	ALTER TABLE hdSettings DROP COLUMN EmailTemplate
	ALTER TABLE hdSettings DROP COLUMN EmailSubjectTemplate
	ALTER TABLE hdSettings DROP COLUMN NewIssueEmailSubject
	ALTER TABLE hdSettings DROP COLUMN NewIssueEmailTemplate
end

-- add fulltext search indexes
-- if fulltext components are installed
declare @installed sql_variant
select @installed=SERVERPROPERTY('IsFullTextInstalled')
IF(@installed = 1)
begin
	-- creating default catalog
	IF NOT EXISTS (SELECT * FROM sys.fulltext_catalogs WHERE name='hdFulltextCatalog')
	BEGIN
		CREATE FULLTEXT CATALOG hdFulltextCatalog AS DEFAULT
	end

	-- creating hdComments index
	IF NOT EXISTS (select * from sys.fulltext_indexes where object_id=object_id('hdComments'))
	BEGIN
	declare @commentsConstraint sysname
	declare @commentIndexCmd varchar(1000)

	SELECT @commentsConstraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='hdComments' AND CONSTRAINT_TYPE='PRIMARY KEY'

	SET @commentIndexCmd = 'CREATE FULLTEXT INDEX ON hdComments ([Body])
							KEY INDEX ' + @commentsConstraint
						
	EXEC(@commentIndexCmd)
	end

	-- creating hdIssues index
	IF NOT EXISTS (select * from sys.fulltext_indexes where object_id=object_id('hdIssues'))
	BEGIN
	declare @issuesConstraint sysname
	declare @issueIndexCmd varchar(1000)

	SELECT @issuesConstraint=CONSTRAINT_NAME FROM INFORMATION_SCHEMA.TABLE_CONSTRAINTS WHERE TABLE_NAME='hdIssues' AND CONSTRAINT_TYPE='PRIMARY KEY'

	SET @issueIndexCmd = 'CREATE FULLTEXT INDEX ON hdIssues ([Subject], [Body])
							KEY INDEX ' + @issuesConstraint
						
	EXEC(@issueIndexCmd)
	end
end

-- user greeting template
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Greeting')
begin
	ALTER TABLE hdUsers ADD Greeting varchar(500) null
end

--user department column
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='Department')
begin
	  ALTER TABLE hdUsers ADD Department nvarchar(500) NOT NULL DEFAULT ''
end

--user is manager column
if not exists (select * from syscolumns where id=object_id('hdUsers') and name='IsManager')
begin
	  ALTER TABLE hdUsers ADD IsManager bit NOT NULL DEFAULT 0
end

--fixing department column
ALTER TABLE hdUsers ALTER COLUMN Department nvarchar(500) NULL

--adding index to hdComments - UserID
if not exists (select name FROM sysindexes WHERE name = 'IX_hdComments_UserID') begin
	CREATE INDEX IX_hdComments_UserID ON hdComments (UserID)
end

--adding index to hdIssues - AssignedToUserID
if not exists (select name FROM sysindexes WHERE name = 'IX_hdIssues_AssignedToUserID') begin
	CREATE INDEX IX_hdIssues_AssignedToUserID ON hdIssues (AssignedToUserID)
end

--automation
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'AutomationRules') AND type in (N'U'))
begin
	CREATE TABLE AutomationRules (
		RuleID int IDENTITY (1, 1) NOT NULL PRIMARY KEY,
		InstanceID int NOT NULL,
		RuleXml varchar(4000)
	)
	CREATE TABLE OverdueAutomationLog (
		IssueID int NOT NULL,
		RuleID int NOT NULL
	)
	ALTER TABLE OverdueAutomationLog WITH NOCHECK ADD 
		CONSTRAINT [PK_OverdueAutomationLog] PRIMARY KEY CLUSTERED (IssueID, RuleID)

	CREATE TABLE NonupdatedAutomationLog (
		IssueID int NOT NULL,
		RuleID int NOT NULL
	)
	ALTER TABLE NonupdatedAutomationLog WITH NOCHECK ADD 
		CONSTRAINT [PK_NonupdatedAutomationLog] PRIMARY KEY CLUSTERED (IssueID, RuleID)
end

--moving KB-tickets to separate table
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'KBArticles') AND type in (N'U'))
begin
	CREATE TABLE [dbo].[KBArticles] (
		[ArticleID] [int] NOT NULL PRIMARY KEY,
		[InstanceID] int NOT NULL DEFAULT(0),
		[CategoryID] [int] NOT NULL ,
		[Subject] [nvarchar] (1000) NOT NULL ,
		[Body] nvarchar(max) NOT NULL,
		[ForTechsOnly] [bit] NOT NULL DEFAULT (0)
	)

	CREATE TABLE [dbo].[KBArticleTags] (
		TagID int NOT NULL,
		ArticleID int NOT NULL
	)
	ALTER TABLE KBArticleTags WITH NOCHECK ADD CONSTRAINT [PK_KBArticleTags] PRIMARY KEY CLUSTERED (TagID,ArticleID)

	--moving kb-tickets to articles
	INSERT INTO KBArticles (ArticleID, InstanceID, CategoryID, [Subject], Body, ForTechsOnly)
		SELECT IssueID, InstanceID, CategoryID, [Subject], Body, KBForTechsOnly FROM hdIssues WHERE PublishToKb=1

	--now moving comments to KB article body
	UPDATE KBArticles
	SET Body = Body + char(10) + ISNULL('[hr]' + (SELECT body + char(10) + char(10) AS [text()] From hdComments where issueID = KBArticles.ArticleID AND IsSystem=0 AND HiddenFromKB=0 AND Body<>'' For XML PATH ('')), '')

	--tags
	INSERT INTO KBArticleTags (ArticleID, TagID)
		SELECT IssueID, TagID FROM hdIssueTags WHERE IssueID IN (SELECT IssueID FROM hdIssues WHERE PublishToKb=1)
end

IF EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'FK_hdFileAttachments_hdIssues') AND parent_object_id = OBJECT_ID(N'hdFileAttachments'))
	ALTER TABLE hdFileAttachments DROP CONSTRAINT [FK_hdFileAttachments_hdIssues]

--add emaildomain to companies
if not exists(select name from syscolumns where id=object_id('hdCompanies') and name='EmailDomain')
	alter table hdCompanies add EmailDomain nvarchar(255) NULL

--custom field categories
IF NOT EXISTS (SELECT * FROM sysobjects WHERE id = OBJECT_ID(N'CustomFieldCategories') AND type in (N'U'))
begin
	CREATE TABLE CustomFieldCategories (
		FieldID int NOT NULL,
		CategoryID int NOT NULL
	)
	ALTER TABLE CustomFieldCategories WITH NOCHECK ADD CONSTRAINT [PK_CustomFieldCategories] PRIMARY KEY CLUSTERED (
		FieldID,
		CategoryID
	)
end

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='CustomCSS')
	alter table hdSettings add CustomCSS varchar(max) NULL

if not exists(select name from syscolumns where id=object_id('hdCategories') and name='FromAddress')
	alter table hdCategories add FromAddress varchar(255) NULL

--drop users foreign key contraints from hdFileAttachments
IF EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'FK_hdFileAttachments_hdUsers') AND parent_object_id = OBJECT_ID(N'hdFileAttachments'))
	ALTER TABLE hdFileAttachments DROP CONSTRAINT [FK_hdFileAttachments_hdUsers]

GO

if not exists(select name from syscolumns where id=object_id('hdUsers') and name='TechTutorialShown')
begin
	alter table hdUsers add TechTutorialShown bit not null default(0)
	
	declare @sql nvarchar(max)
	set @sql = 'update hdUsers SET TechTutorialShown=1'
	exec sp_executesql @sql
end

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='DisableAutoTimer')
	alter table hdSettings add DisableAutoTimer bit not null default 0

if not exists(select name from syscolumns where id=object_id('hdComments') and name='EmailHeaders')
	alter table hdComments add EmailHeaders varchar(1000) NULL

if not exists(select name from syscolumns where id=object_id('hdUsers') and name='PushToken')
	alter table hdUsers add PushToken varchar(64) NULL

if not exists(select name from syscolumns where id=object_id('hdSettings') and name='DontAddCCToSubscribers')
	alter table hdSettings add DontAddCCToSubscribers bit not null default 0

if not exists(select name from syscolumns where id=object_id('hdCompanies') and name='Notes')
	alter table hdCompanies add Notes nvarchar(2000) NULL

if not exists(select name from syscolumns where id=object_id('Instances') and name='AvangateSubscriptionID')
	alter table Instances add AvangateSubscriptionID nvarchar(20) NULL

if not exists (select * from syscolumns where id=object_id('KBArticles') and name='OrderByNumber')
	ALTER TABLE KBArticles ADD OrderByNumber int NOT NULL DEFAULT 1


--creating cat index in KB
if not exists (select name FROM sysindexes WHERE name = 'IX_KBArticles_CategoryID')
	CREATE INDEX IX_KBArticles_CategoryID ON KBArticles (CategoryID)